@@ -4,7 +4,7 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
                                        geo.dist = 0.1, tls.resolution = list(), tls.precision = NULL,
                                        stem.section = c(0.7, 3.5), stem.range = NULL, breaks = NULL,
                                        slice = 0.1, understory = NULL, bark.roughness = 1,
-                                       den.type = 1, d.top = NULL,
+                                       den.type = 1, d.mer = NULL,
                                        segmentation = NULL,
                                        plot.attributes = NULL, plot = TRUE,
                                        threads = 1,
@@ -108,13 +108,6 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
   noise <- noise[, c("id", "file", "x", "y", "z", "rho")]
 
 
-  # 2. Defining the vertical section in which trees are detected
-  # Applying +/- geo.dist as a buffer to compute geometric features properly
-
-
-  stem <- woody[woody$z > stem.section[1] - geo.dist & woody$z < stem.section[2] + geo.dist, ]
-
-
 
   # 3. Computing geometric features
 
@@ -122,77 +115,53 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
 
   threads <- max(1, threads)
 
-  # VerSur <- geometric.features(data = stem,
-  #                              approximate_KNN = FALSE,
-  #                              features = c("verticality", "surface_variation", "planarity"),
-  #                              dist = geo.dist,
-  #                              threads = threads,
-  #                              keep_NaN = FALSE,            # this means, when we run the Rcpp code we don't ex <- ude computed rows if 1 of the features is NA. If we have to compute 13 features and 1 is NA, then we keep this row
-  #                              verbose = TRUE,
-  #                              solver_threshold = 50000)
-  #
-  # if(is.null(VerSur$verticality) | is.null(VerSur$surface_variation) | is.null(VerSur$planarity)){
-  #
-  #   VerSur$verticality <- NA
-  #   VerSur$surface_variation <- NA
-  #   VerSur$planarity <- NA
-  #
-  # }
+  GeometricFeatures <- geometric_features_dist(points = as.matrix(woody[, c("point", "x", "y", "z")]),
+                                               dist = geo.dist,
+                                               Planarity = TRUE,
+                                               Surface_variation = TRUE,
+                                               Verticality = TRUE,
+                                               num_threads = threads)
 
-  VerSur <- geometric_features_py(data = stem,
-                                  dist = geo.dist,
-                                  threads = as.integer(threads))
+  # GeometricFeatures <- geometric_features_py(data = stem,
+  #                                            dist = geo.dist,
+  #                                            threads = as.integer(threads))
 
 
   # Retaining those points within the vertical section defined in the arguments
 
-  stem <- stem[stem$z > stem.section[1] & stem$z < stem.section[2], ]
+  woody <- merge(woody, GeometricFeatures[, c("point", "Planarity", "Surface_variation", "Verticality")], by = "point")
 
-  stem <- merge(stem, VerSur[, c("point", "verticality", "surface_variation", "planarity")], by = "point")
-
-  rm(VerSur)
+  rm(GeometricFeatures)
 
 
 
   # 4. Retention of points with high verticality
 
-  stem$ver <- stem$verticality
-  stem$ver <- ifelse(is.na(stem$ver), stats::runif(1), stem$ver)
-
-  stem$prob.ver <- stats::runif(nrow(stem), min = 0, max = 1)
-  stem <- stem[stem$ver > stem$prob.ver, ]
-
-
-  # Keeping only points with high verticality, planarity & low surface variation
-  # within the vertical section defined in the arguments in the whole point cloud
-
-  woody <- woody[woody$z <= stem.section[1] | woody$z >= stem.section[2], ]
-  woody <- rbind(woody, stem[, 1:ncol(woody)])
+  woody <- woody[woody$Verticality > 0.7, ]
 
 
   # 5. Retention of points with low surface variation
 
-  stem$ver <- stem$surface_variation / 0.33
-  stem$ver <- ifelse(is.na(stem$ver), stats::runif(1), stem$ver)
-
-  stem$prob.ver <- stats::runif(nrow(stem), min = 0, max = 1)
-  stem <- stem[stem$ver < stem$prob.ver, ]
+  woody$Surface_variation <- woody$Surface_variation / 0.33
+  woody$Surface_variation <- ifelse(is.na(woody$Surface_variation), stats::runif(1), woody$Surface_variation)
+  woody$prob.ver <- stats::runif(nrow(woody), min = 0, max = 1)
+  woody <- woody[woody$Surface_variation < woody$prob.ver, ]
+  # woody <- woody[woody$Surface_variation < 0.95, ]
 
 
   # 6. Retention of points with high planarity
 
-  stem$ver <- stem$planarity
-  stem$ver <- ifelse(is.na(stem$ver), stats::runif(1), stem$ver)
-
-  stem$prob.ver <- stats::runif(nrow(stem), min = 0, max = 1)
-  stem <- stem[stem$ver > stem$prob.ver, ]
+  # woody$Planarity <- ifelse(is.na(woody$Planarity), stats::runif(1), woody$Planarity)
+  woody <- woody[woody$Planarity > 0.05, ]
 
 
   # 7. Retention of high points
 
-  # stem$ver <- (stem$z - stem.section[1]) / (stem.section[2] - stem.section[1])
-  # stem$prob.ver <- stats::runif(nrow(stem), min = 0, max = 1)
-  # stem <- stem[stem$ver > stem$prob.ver, ]
+  stem <- woody[woody$z > stem.section[1] & woody$z < stem.section[2], c("x", "y", "z", "prob.selec")]
+
+  stem$ver <- (stem$z - stem.section[1]) / (stem.section[2] - stem.section[1])
+  stem$prob.ver <- stats::runif(nrow(stem), min = 0, max = 1)
+  stem <- stem[stem$ver > stem$prob.ver, ]
 
 
 
@@ -235,7 +204,7 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
 
     warning("No tree was detected")
 
-    .tree <- .no.trees.detected.multi(data, d.top, plot.attributes, dir.result, save.result)
+    .tree <- .no.trees.detected.multi(data, d.mer, plot.attributes, dir.result, save.result)
     return(.tree)
 
   }
@@ -368,71 +337,11 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
     cuts <- breaks[i]
 
 
-    .cut <- woody[woody$z > (cuts - 2 * slice - geo.dist) & woody$z < cuts + geo.dist, , drop = FALSE]
-
-
-    if(nrow(.cut) < 50){next}
-
-
-    if(cuts <= stem.section[1] | cuts >= stem.section[2]){
-
-      # VerSur <- geometric.features(data = .cut,
-      #                              approximate_KNN = FALSE,
-      #                              features = c("verticality", "surface_variation", "planarity"),
-      #                              dist = geo.dist,
-      #                              threads = threads,
-      #                              keep_NaN = FALSE,            # this means, when we run the Rcpp code we don't exclude computed rows if 1 of the features is NA. If we have to compute 13 features and 1 is NA, then we keep this row
-      #                              verbose = FALSE,
-      #                              solver_threshold = 50000)
-      #
-      # if(is.null(VerSur$verticality) | is.null(VerSur$surface_variation) | is.null(VerSur$planarity)){
-      #
-      #   VerSur$verticality <- NA
-      #   VerSur$surface_variation <- NA
-      #   VerSur$planarity <- NA
-      #
-      # }
-
-      VerSur <- geometric_features_py(data = .cut,
-                                      dist = geo.dist,
-                                      threads = as.integer(threads))
-
-
-      VerSur <- VerSur[!duplicated(VerSur$point), ]
-
-      .cut <- merge(.cut, VerSur[, c("point", "verticality", "surface_variation", "planarity")], by = "point")
-
-      rm(VerSur)
-
-      .cut$ver <- .cut$verticality
-      .cut$ver <- ifelse(is.na(.cut$ver), stats::runif(1), .cut$ver)
-
-      .cut$prob.ver <- stats::runif(nrow(.cut), min = 0, max = 1)
-      .cut <- .cut[.cut$ver > .cut$prob.ver, ]
-
-
-      # .cut$ver <- .cut$surface_variation / 0.33
-      # .cut$ver <- ifelse(is.na(.cut$ver), stats::runif(1), .cut$ver)
-      #
-      # .cut$prob.ver <- stats::runif(nrow(.cut), min = 0, max = 1)
-      # .cut <- .cut[.cut$ver < .cut$prob.ver, ]
-      #
-      #
-      # .cut$ver <- .cut$planarity
-      # .cut$ver <- ifelse(is.na(.cut$ver), stats::runif(1), .cut$ver)
-      #
-      # .cut$prob.ver <- stats::runif(nrow(.cut), min = 0, max = 1)
-      # .cut <- .cut[.cut$ver > .cut$prob.ver, ]
-
-    }
+    .cut <- woody[woody$z > (cuts - 2 * slice) & woody$z < cuts, , drop = FALSE]
 
     if(nrow(.cut) < 25){next}
 
 
-    # Restrict to slice corresponding to cuts m +/- 5 cm
-
-    .cut <- .cut[.cut$z > (cuts - 2 * slice) & .cut$z < cuts, , drop = FALSE]
-
 
     # DBSCAN parameters
 
@@ -489,56 +398,11 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
 
     if(!is.null(stem.2)){
 
-      .cut <- stem.2[stem.2$z > (cuts - 2 * slice - geo.dist) & stem.2$z < cuts + geo.dist, , drop = FALSE]
-
-
-      if(nrow(.cut) < 50){next}
-
-
-      if(cuts <= stem.section[1] | cuts >= stem.section[2]){
-
-        VerSur <- geometric.features(data = .cut,
-                                     approximate_KNN = FALSE,
-                                     features = c("verticality", "surface_variation"),
-                                     dist = geo.dist,
-                                     threads = threads,
-                                     keep_NaN = FALSE,            # this means, when we run the Rcpp code we don't exclude computed rows if 1 of the features is NA. If we have to compute 13 features and 1 is NA, then we keep this row
-                                     verbose = FALSE,
-                                     solver_threshold = 50000)
-
-        if(is.null(VerSur$verticality) | is.null(VerSur$surface_variation)){
-
-          VerSur$verticality <- NA
-          VerSur$surface_variation <- NA
-
-        }
-
-        .cut <- merge(.cut, VerSur[, c("point", "verticality", "surface_variation")], by = "point")
-
-        rm(VerSur)
-
-        .cut$ver <- .cut$verticality
-        .cut$ver <- ifelse(is.na(.cut$ver), stats::runif(1), .cut$ver)
-
-        .cut$prob.ver <- stats::runif(nrow(.cut), min = 0, max = 1)
-        .cut <- .cut[.cut$ver > 0.75 | .cut$ver > .cut$prob.ver, ]
-
-
-        .cut$ver <- .cut$surface_variation / 0.33
-        .cut$ver <- ifelse(is.na(.cut$ver), stats::runif(1), .cut$ver)
-
-        .cut$prob.ver <- stats::runif(nrow(.cut), min = 0, max = 1)
-        .cut <- .cut[.cut$ver < .cut$prob.ver, ]
-
-      }
+      .cut <- woody[woody$z > (cuts - 2 * slice) & woody$z < cuts, , drop = FALSE]
 
       if(nrow(.cut) < 25){next}
 
 
-      # Restrict to slice corresponding to cuts m +/- 5 cm
-
-      .cut <- .cut[.cut$z > (cuts - 2 * slice) & .cut$z < cuts, , drop = FALSE]
-
       # DBSCAN parameters
 
       .eps <- 2 * (tan(.alpha.h / 2) * (max(.cut$r, na.rm = TRUE) / cos(mean(.cut$slope, na.rm = TRUE))) * 2)
@@ -595,7 +459,7 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
 
     warning("No tree was detected")
 
-    .tree <- .no.trees.detected.single(data, d.top, plot.attributes, dir.result, save.result)
+    .tree <- .no.trees.detected.single(data, d.mer, plot.attributes, dir.result, save.result)
     return(.tree)
 
   }
@@ -633,7 +497,7 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
 
     warning("No tree was detected")
 
-    .tree <- .no.trees.detected.single(data, d.top, plot.attributes, dir.result, save.result)
+    .tree <- .no.trees.detected.single(data, d.mer, plot.attributes, dir.result, save.result)
     return(.tree)
 
     }
@@ -781,7 +645,7 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
 
       warning("No tree was detected")
 
-      .tree <- .no.trees.detected.single(data, d.top, plot.attributes, dir.result, save.result)
+      .tree <- .no.trees.detected.single(data, d.mer, plot.attributes, dir.result, save.result)
       return(.tree)
 
     }
@@ -827,7 +691,7 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
 
       warning("No tree was detected")
 
-      .tree <- .no.trees.detected.single(data, d.top, plot.attributes, dir.result, save.result)
+      .tree <- .no.trees.detected.single(data, d.mer, plot.attributes, dir.result, save.result)
       return(.tree)
 
     }
@@ -842,7 +706,7 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
 
       warning("No tree was detected")
 
-      .tree <- .no.trees.detected.single(data, d.top, plot.attributes, dir.result, save.result)
+      .tree <- .no.trees.detected.single(data, d.mer, plot.attributes, dir.result, save.result)
       return(.tree)
 
     }
@@ -858,7 +722,7 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
 
       warning("No tree was detected")
 
-      .tree <- .no.trees.detected.single(data, d.top, plot.attributes, dir.result, save.result)
+      .tree <- .no.trees.detected.single(data, d.mer, plot.attributes, dir.result, save.result)
       return(.tree)
 
     }
@@ -873,7 +737,7 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
 
       warning("No tree was detected")
 
-      .tree <- .no.trees.detected.single(data, d.top, plot.attributes, dir.result, save.result)
+      .tree <- .no.trees.detected.single(data, d.mer, plot.attributes, dir.result, save.result)
       return(.tree)
 
     }
@@ -1112,29 +976,29 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
 
     # Compute volume (m3)
 
-    if(length(table(.stem$hi)) > 3 & is.null(d.top)){
+    if(length(table(.stem$hi)) > 3 & is.null(d.mer)){
 
       # Stem curve
 
       stem.v <- .volume(.stem, id = data$id[1], den.type = den.type)
       .tree <- merge(.tree, stem.v, all = TRUE)
 
-    } else if (length(table(.stem$hi)) > 3 & !is.null(d.top)) {
+    } else if (length(table(.stem$hi)) > 3 & !is.null(d.mer)) {
 
-      stem.v <- .volume(.stem, d.top, id = data$id[1], den.type = den.type)
+      stem.v <- .volume(.stem, d.mer, id = data$id[1], den.type = den.type)
       .tree <- merge(.tree, stem.v, all = TRUE)
 
 
-    } else if (length(table(.stem$hi)) <= 3 & !is.null(d.top)) {
+    } else if (length(table(.stem$hi)) <= 3 & !is.null(d.mer)) {
 
       n <- den.type
       # Paraboloid volume
 
       .tree$v <- pi * (.tree[, "h"] ^ (n + 1) / (n + 1)) * ((.tree[, "dbh"] / 200) ^ 2 / (.tree[, "h"] - 1.3) ^ n)
-      h.lim <- (((d.top / 200) ^ 2) / ((.tree[, "dbh"] / 200) ^ 2 / (.tree[, "h"] - 1.3) ^ n)) ^ (1 / n)
-      .tree$v.com <- pi * ((.tree[, "h"] ^ (n + 1) - h.lim ^ (n + 1)) / (n + 1)) * ((.tree[, "dbh"] / 200) ^ 2 / (.tree[, "h"] - 1.3) ^ n)
-      .tree$v.com <- ifelse(.tree$v.com < 0, 0, .tree$v.com)
-      .tree$h.com <- h.lim
+      h.lim <- (((d.mer / 200) ^ 2) / ((.tree[, "dbh"] / 200) ^ 2 / (.tree[, "h"] - 1.3) ^ n)) ^ (1 / n)
+      .tree$v.mer <- pi * ((.tree[, "h"] ^ (n + 1) - h.lim ^ (n + 1)) / (n + 1)) * ((.tree[, "dbh"] / 200) ^ 2 / (.tree[, "h"] - 1.3) ^ n)
+      .tree$v.mer <- ifelse(.tree$v.mer < 0, 0, .tree$v.mer)
+      .tree$h.mer <- h.lim
 
     } else {
 
@@ -1173,18 +1037,18 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
 
 
     # If plot identification (id) is not available
-    if(is.null(data$id) & is.null(.tree$v.com)){
+    if(is.null(data$id) & is.null(.tree$v.mer)){
 
       .tree <- .tree[, c("tree", "x", "y", "phi", "phi.left", "phi.right", "horizontal.distance", "dbh", "h", "v", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion"), drop = FALSE]
       colnames(.tree) <- c("tree", "x", "y", "phi", "phi.left", "phi.right", "h.dist", "dbh", "h", "v", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion")
 
-    } else if (is.null(data$id) & !is.null(.tree$v.com)) {
+    } else if (is.null(data$id) & !is.null(.tree$v.mer)) {
 
-      .tree <- .tree[, c("tree", "x", "y", "phi", "phi.left", "phi.right", "horizontal.distance", "dbh", "h", "h.com", "v", "v.com", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion"), drop = FALSE]
-      colnames(.tree) <- c("tree", "x", "y", "phi", "phi.left", "phi.right", "h.dist", "dbh", "h", "h.com", "v", "v.com", "n.pts", "SS.max", "sinuosity", "lean", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion")
+      .tree <- .tree[, c("tree", "x", "y", "phi", "phi.left", "phi.right", "horizontal.distance", "dbh", "h", "h.mer", "v", "v.mer", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion"), drop = FALSE]
+      colnames(.tree) <- c("tree", "x", "y", "phi", "phi.left", "phi.right", "h.dist", "dbh", "h", "h.mer", "v", "v.mer", "n.pts", "SS.max", "sinuosity", "lean", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion")
 
 
-    } else if (!is.null(data$id) & is.null(.tree$v.com)) {
+    } else if (!is.null(data$id) & is.null(.tree$v.mer)) {
 
       # If plot identification (id) is available
 
@@ -1201,8 +1065,8 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
       .tree$id <- data$id[1]
       .tree$file <- data$file[1]
 
-      .tree <- .tree[, c("id", "file", "tree", "x", "y", "phi", "phi.left", "phi.right", "horizontal.distance", "dbh", "h", "h.com", "v", "v.com", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion"), drop = FALSE]
-      colnames(.tree) <- c("id", "file", "tree", "x", "y", "phi", "phi.left", "phi.right", "h.dist", "dbh", "h", "h.com", "v", "v.com", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion")
+      .tree <- .tree[, c("id", "file", "tree", "x", "y", "phi", "phi.left", "phi.right", "horizontal.distance", "dbh", "h", "h.mer", "v", "v.mer", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion"), drop = FALSE]
+      colnames(.tree) <- c("id", "file", "tree", "x", "y", "phi", "phi.left", "phi.right", "h.dist", "dbh", "h", "h.mer", "v", "v.mer", "SS.max", "sinuosity", "lean", "n.pts", "n.pts.red", "n.pts.est", "n.pts.red.est", "partial.occlusion")
 
     }
 
